@@ -28,7 +28,11 @@ typedef struct
   fe25519 t;
 } ge25519;
 
-const ge25519 ge25519_base;
+#ifdef __cplusplus
+extern "C" {
+#endif
+
+extern const ge25519 ge25519_base;
 
 int ge25519_unpackneg_vartime(ge25519 *r, const unsigned char p[32]);
 
@@ -40,4 +44,8 @@ void ge25519_double_scalarmult_vartime(ge25519 *r, const ge25519 *p1, const sc25
 
 void ge25519_scalarmult_base(ge25519 *r, const sc25519 *s);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif
