@@ -7,6 +7,10 @@
 #define POLY1305_H
 #include "libssh/chacha20-poly1305-common.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 void poly1305_auth(uint8_t out[POLY1305_TAGLEN], const uint8_t *m, size_t inlen,
     const uint8_t key[POLY1305_KEYLEN])
 #ifdef HAVE_GCC_BOUNDED_ATTRIBUTE
@@ -16,4 +20,8 @@ void poly1305_auth(uint8_t out[POLY1305_TAGLEN], const uint8_t *m, size_t inlen,
 #endif
     ;
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif	/* POLY1305_H */
