@@ -12,9 +12,6 @@
 
 enum pkd_hostkey_type_e {
     PKD_RSA,
-#ifdef HAVE_DSA
-    PKD_DSA,
-#endif
     PKD_ED25519,
     PKD_ECDSA
 };
@@ -44,10 +41,12 @@ struct pkd_daemon_args {
         unsigned int iterations;
 
         struct {
+            const char *argv_mkdtemp_str;
             char *mkdtemp_str;
         } socket_wrapper;
 
         struct {
+            const char *argv_mkdtemp_str;
             char *mkdtemp_str;
         } temp_dir;
     } opts;
