@@ -9,15 +9,10 @@
 
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
-#elif (defined _WIN32) || (defined _WIN64)
-#include <io.h>
-#define read _read
-#define open _open
-#define write _write
-#define close _close
 #endif
 
 #include "torture_pki.h"
+#include <libssh/priv.h>
 
 char *torture_pki_read_file(const char *filename)
 {
