@@ -1,14 +1,15 @@
 #include "config.h"
+#include "libssh/libssh.h"
 
 #define LIBSSH_STATIC
 
 #include <sys/stat.h>
 #include <fcntl.h>
 
+#include "pki.c"
 #include "torture.h"
 #include "torture_key.h"
 #include "torture_pki.h"
-#include "pki.c"
 
 #define LIBSSH_ECDSA_TESTKEY "libssh_testkey.id_ecdsa"
 #define LIBSSH_ECDSA_TESTKEY_PASSPHRASE "libssh_testkey_passphrase.id_ecdsa"
@@ -218,27 +219,60 @@ static void torture_pki_ecdsa_import_pubkey_from_openssh_privkey(void **state)
     SSH_KEY_FREE(pubkey);
 }
 
-static void torture_pki_ecdsa_import_privkey_base64(void **state)
+static void
+torture_pki_ecdsa_import_export_privkey_base64_format(void **state,
+                                                      enum ssh_file_format_e format)
 {
     int rc;
-    char *key_str = NULL;
-    ssh_key key = NULL;
+    char *key_str = NULL, *new_key_str = NULL;
+    ssh_key key = NULL, new_key = NULL;
     const char *passphrase = torture_get_testkey_passphrase();
 
-    (void) state; /* unused */
+    (void)state; /* unused */
 
     key_str = torture_pki_read_file(LIBSSH_ECDSA_TESTKEY);
     assert_non_null(key_str);
 
     rc = ssh_pki_import_privkey_base64(key_str, passphrase, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(key);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
+
+    /* Export */
+    rc = ssh_pki_export_privkey_base64_format(key,
+                                              passphrase,
+                                              NULL,
+                                              NULL,
+                                              &new_key_str,
+                                              format);
+    assert_int_equal(rc, SSH_OK);
+    assert_non_null(new_key_str);
+
+    /* and import again */
+    rc = ssh_pki_import_privkey_base64(new_key_str, passphrase, NULL, NULL,
+                                       &new_key);
+    assert_int_equal(rc, 0);
+    assert_non_null(new_key);
+
+    rc = ssh_key_is_private(new_key);
+    assert_int_equal(rc, 1);
+
+    rc = ssh_key_cmp(key, new_key, SSH_KEY_CMP_PRIVATE | SSH_KEY_CMP_PUBLIC);
+    assert_int_equal(rc, 0);
 
     free(key_str);
+    free(new_key_str);
     SSH_KEY_FREE(key);
+    SSH_KEY_FREE(new_key);
+}
+
+static void
+torture_pki_ecdsa_import_export_privkey_base64_default(void **state)
+{
+    torture_pki_ecdsa_import_export_privkey_base64_format(state,
+                                                          SSH_FILE_FORMAT_DEFAULT);
 }
 
 static void torture_pki_ecdsa_import_privkey_base64_comment(void **state)
@@ -261,11 +295,11 @@ static void torture_pki_ecdsa_import_privkey_base64_comment(void **state)
     assert_int_equal(rc, file_str_len - 1);
 
     rc = ssh_pki_import_privkey_base64(file_str, passphrase, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(key);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     free(key_str);
     free(file_str);
@@ -292,11 +326,11 @@ static void torture_pki_ecdsa_import_privkey_base64_whitespace(void **state)
     assert_int_equal(rc, file_str_len - 1);
 
     rc = ssh_pki_import_privkey_base64(file_str, passphrase, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(key);
 
     rc = ssh_key_is_private(key);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     free(key_str);
     free(file_str);
@@ -318,11 +352,11 @@ static void torture_pki_ecdsa_publickey_from_privatekey(void **state)
     assert_non_null(key_str);
 
     rc = ssh_pki_import_privkey_base64(key_str, passphrase, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(key);
 
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey);
 
     free(key_str);
@@ -337,15 +371,20 @@ static void torture_pki_ecdsa_import_cert_file(void **state)
     enum ssh_keytypes_e type;
     struct pki_st *test_state = *((struct pki_st **)state);
 
+    /* Importing public key as cert should fail */
+    rc = ssh_pki_import_cert_file(LIBSSH_ECDSA_TESTKEY ".pub", &cert);
+    assert_int_equal(rc, SSH_ERROR);
+    assert_null(cert);
+
     rc = ssh_pki_import_cert_file(LIBSSH_ECDSA_TESTKEY "-cert.pub", &cert);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(cert);
 
     type = ssh_key_type(cert);
-    assert_true(type == test_state->type+3);
+    assert_int_equal(type, test_state->type+3);
 
     rc = ssh_key_is_public(cert);
-    assert_true(rc == 1);
+    assert_int_equal(rc, 1);
 
     SSH_KEY_FREE(cert);
 }
@@ -369,7 +408,7 @@ static void torture_pki_ecdsa_publickey_base64(void **state)
     }
 
     type = ssh_key_type_from_name(q);
-    assert_true(type == test_state->type);
+    assert_int_equal(type, test_state->type);
 
     q = ++p;
     while (p != NULL && *p != '\0' && *p != ' ') p++;
@@ -378,11 +417,11 @@ static void torture_pki_ecdsa_publickey_base64(void **state)
     }
 
     rc = ssh_pki_import_pubkey_base64(q, type, &key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(key);
 
     rc = ssh_pki_export_pubkey_base64(key, &b64_key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(b64_key);
 
     assert_string_equal(q, b64_key);
@@ -406,7 +445,7 @@ static void torture_pki_ecdsa_generate_pubkey_from_privkey(void **state)
     rc = torture_read_one_line(LIBSSH_ECDSA_TESTKEY ".pub",
                                pubkey_original,
                                sizeof(pubkey_original));
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
 
     /* remove the public key, generate it from the private key and write it. */
     unlink(LIBSSH_ECDSA_TESTKEY ".pub");
@@ -416,22 +455,23 @@ static void torture_pki_ecdsa_generate_pubkey_from_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(privkey);
 
     rc = ssh_pki_export_privkey_to_pubkey(privkey, &pubkey);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey);
 
     rc = ssh_pki_export_pubkey_file(pubkey, LIBSSH_ECDSA_TESTKEY ".pub");
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
 
     rc = torture_read_one_line(LIBSSH_ECDSA_TESTKEY ".pub",
                                pubkey_generated,
                                sizeof(pubkey_generated));
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     len = torture_pubkey_len(pubkey_original);
-    assert_int_equal(strncmp(pubkey_original, pubkey_generated, len), 0);
+    assert_int_equal(len, torture_pubkey_len(pubkey_generated));
+    assert_memory_equal(pubkey_original, pubkey_generated, len);
 
     SSH_KEY_FREE(privkey);
     SSH_KEY_FREE(pubkey);
@@ -450,11 +490,11 @@ static void torture_pki_ecdsa_duplicate_key(void **state)
     (void) state;
 
     rc = ssh_pki_import_pubkey_file(LIBSSH_ECDSA_TESTKEY ".pub", &pubkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(pubkey);
 
     rc = ssh_pki_export_pubkey_base64(pubkey, &b64_key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(b64_key);
 
     rc = ssh_pki_import_privkey_file(LIBSSH_ECDSA_TESTKEY,
@@ -462,27 +502,27 @@ static void torture_pki_ecdsa_duplicate_key(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(privkey);
 
     privkey_dup = ssh_key_dup(privkey);
     assert_non_null(privkey_dup);
 
     rc = ssh_pki_export_privkey_to_pubkey(privkey, &pubkey_dup);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(pubkey_dup);
 
     rc = ssh_pki_export_pubkey_base64(pubkey_dup, &b64_key_gen);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(b64_key_gen);
 
     assert_string_equal(b64_key, b64_key_gen);
 
     rc = ssh_key_cmp(privkey, privkey_dup, SSH_KEY_CMP_PRIVATE);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
 
     rc = ssh_key_cmp(pubkey, pubkey_dup, SSH_KEY_CMP_PUBLIC);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
 
     SSH_KEY_FREE(pubkey);
     SSH_KEY_FREE(pubkey_dup);
@@ -539,7 +579,7 @@ static void torture_pki_generate_key_ecdsa(void **state)
     (void) state;
 
     rc = ssh_pki_generate(SSH_KEYTYPE_ECDSA_P256, 0, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -547,13 +587,13 @@ static void torture_pki_generate_key_ecdsa(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA256);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_ECDSA_P256);
+    assert_int_equal(type, SSH_KEYTYPE_ECDSA_P256);
     type_char = ssh_key_type_to_char(type);
-    assert_true(strcmp(type_char, "ecdsa-sha2-nistp256") == 0);
+    assert_string_equal(type_char, "ecdsa-sha2-nistp256");
     etype_char = ssh_pki_key_ecdsa_name(key);
-    assert_true(strcmp(etype_char, "ecdsa-sha2-nistp256") == 0);
+    assert_string_equal(etype_char, "ecdsa-sha2-nistp256");
 
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
@@ -561,7 +601,7 @@ static void torture_pki_generate_key_ecdsa(void **state)
 
     /* deprecated */
     rc = ssh_pki_generate(SSH_KEYTYPE_ECDSA, 256, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -569,20 +609,20 @@ static void torture_pki_generate_key_ecdsa(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA256);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_ECDSA_P256);
+    assert_int_equal(type, SSH_KEYTYPE_ECDSA_P256);
     type_char = ssh_key_type_to_char(type);
-    assert_true(strcmp(type_char, "ecdsa-sha2-nistp256") == 0);
+    assert_string_equal(type_char, "ecdsa-sha2-nistp256");
     etype_char = ssh_pki_key_ecdsa_name(key);
-    assert_true(strcmp(etype_char, "ecdsa-sha2-nistp256") == 0);
+    assert_string_equal(etype_char, "ecdsa-sha2-nistp256");
 
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
     SSH_KEY_FREE(pubkey);
 
     rc = ssh_pki_generate(SSH_KEYTYPE_ECDSA_P384, 0, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -590,13 +630,13 @@ static void torture_pki_generate_key_ecdsa(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA384);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_ECDSA_P384);
+    assert_int_equal(type, SSH_KEYTYPE_ECDSA_P384);
     type_char = ssh_key_type_to_char(type);
-    assert_true(strcmp(type_char, "ecdsa-sha2-nistp384") == 0);
-    etype_char =ssh_pki_key_ecdsa_name(key);
-    assert_true(strcmp(etype_char, "ecdsa-sha2-nistp384") == 0);
+    assert_string_equal(type_char, "ecdsa-sha2-nistp384");
+    etype_char = ssh_pki_key_ecdsa_name(key);
+    assert_string_equal(etype_char, "ecdsa-sha2-nistp384");
 
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
@@ -604,7 +644,7 @@ static void torture_pki_generate_key_ecdsa(void **state)
 
     /* deprecated */
     rc = ssh_pki_generate(SSH_KEYTYPE_ECDSA, 384, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -612,20 +652,20 @@ static void torture_pki_generate_key_ecdsa(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA384);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_ECDSA_P384);
+    assert_int_equal(type, SSH_KEYTYPE_ECDSA_P384);
     type_char = ssh_key_type_to_char(type);
-    assert_true(strcmp(type_char, "ecdsa-sha2-nistp384") == 0);
-    etype_char =ssh_pki_key_ecdsa_name(key);
-    assert_true(strcmp(etype_char, "ecdsa-sha2-nistp384") == 0);
+    assert_string_equal(type_char, "ecdsa-sha2-nistp384");
+    etype_char = ssh_pki_key_ecdsa_name(key);
+    assert_string_equal(etype_char, "ecdsa-sha2-nistp384");
 
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
     SSH_KEY_FREE(pubkey);
 
     rc = ssh_pki_generate(SSH_KEYTYPE_ECDSA_P521, 0, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -633,13 +673,13 @@ static void torture_pki_generate_key_ecdsa(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA512);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_ECDSA_P521);
+    assert_int_equal(type, SSH_KEYTYPE_ECDSA_P521);
     type_char = ssh_key_type_to_char(type);
-    assert_true(strcmp(type_char, "ecdsa-sha2-nistp521") == 0);
+    assert_string_equal(type_char, "ecdsa-sha2-nistp521");
     etype_char =ssh_pki_key_ecdsa_name(key);
-    assert_true(strcmp(etype_char, "ecdsa-sha2-nistp521") == 0);
+    assert_string_equal(etype_char, "ecdsa-sha2-nistp521");
 
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
@@ -647,7 +687,7 @@ static void torture_pki_generate_key_ecdsa(void **state)
 
     /* deprecated */
     rc = ssh_pki_generate(SSH_KEYTYPE_ECDSA, 521, &key);
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     assert_non_null(key);
     rc = ssh_pki_export_privkey_to_pubkey(key, &pubkey);
     assert_int_equal(rc, SSH_OK);
@@ -655,13 +695,13 @@ static void torture_pki_generate_key_ecdsa(void **state)
     sign = pki_do_sign(key, INPUT, sizeof(INPUT), SSH_DIGEST_SHA512);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, pubkey, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     type = ssh_key_type(key);
-    assert_true(type == SSH_KEYTYPE_ECDSA_P521);
+    assert_int_equal(type, SSH_KEYTYPE_ECDSA_P521);
     type_char = ssh_key_type_to_char(type);
-    assert_true(strcmp(type_char, "ecdsa-sha2-nistp521") == 0);
-    etype_char =ssh_pki_key_ecdsa_name(key);
-    assert_true(strcmp(etype_char, "ecdsa-sha2-nistp521") == 0);
+    assert_string_equal(type_char, "ecdsa-sha2-nistp521");
+    etype_char = ssh_pki_key_ecdsa_name(key);
+    assert_string_equal(etype_char, "ecdsa-sha2-nistp521");
 
     ssh_signature_free(sign);
     SSH_KEY_FREE(key);
@@ -684,11 +724,11 @@ static void torture_pki_ecdsa_cert_verify(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(privkey);
 
     rc = ssh_pki_import_cert_file(LIBSSH_ECDSA_TESTKEY "-cert.pub", &cert);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(cert);
 
     /* Get the hash type to be used in the signature based on the key type */
@@ -697,7 +737,7 @@ static void torture_pki_ecdsa_cert_verify(void **state)
     sign = pki_do_sign(privkey, INPUT, sizeof(INPUT), hash_type);
     assert_non_null(sign);
     rc = ssh_pki_signature_verify(session, sign, cert, INPUT, sizeof(INPUT));
-    assert_true(rc == SSH_OK);
+    assert_return_code(rc, errno);
     ssh_signature_free(sign);
     SSH_KEY_FREE(privkey);
     SSH_KEY_FREE(cert);
@@ -822,8 +862,9 @@ static void torture_pki_fail_sign_with_incompatible_hash(void **state)
     SSH_KEY_FREE(key);
 }
 
-#ifdef HAVE_LIBCRYPTO
-static void torture_pki_ecdsa_write_privkey(void **state)
+static void
+torture_pki_ecdsa_write_privkey_format(void **state,
+                                       enum ssh_file_format_e format)
 {
     ssh_key origkey = NULL;
     ssh_key privkey = NULL;
@@ -836,28 +877,29 @@ static void torture_pki_ecdsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &origkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(origkey);
 
     unlink(LIBSSH_ECDSA_TESTKEY);
 
-    rc = ssh_pki_export_privkey_file(origkey,
-                                     NULL,
-                                     NULL,
-                                     NULL,
-                                     LIBSSH_ECDSA_TESTKEY);
-    assert_true(rc == 0);
+    rc = ssh_pki_export_privkey_file_format(origkey,
+                                            NULL,
+                                            NULL,
+                                            NULL,
+                                            LIBSSH_ECDSA_TESTKEY,
+                                            format);
+    assert_int_equal(rc, 0);
 
     rc = ssh_pki_import_privkey_file(LIBSSH_ECDSA_TESTKEY,
                                      NULL,
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(privkey);
 
     rc = ssh_key_cmp(origkey, privkey, SSH_KEY_CMP_PRIVATE);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
 
     SSH_KEY_FREE(origkey);
     SSH_KEY_FREE(privkey);
@@ -868,16 +910,17 @@ static void torture_pki_ecdsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &origkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(origkey);
 
     unlink(LIBSSH_ECDSA_TESTKEY_PASSPHRASE);
-    rc = ssh_pki_export_privkey_file(origkey,
-                                     torture_get_testkey_passphrase(),
-                                     NULL,
-                                     NULL,
-                                     LIBSSH_ECDSA_TESTKEY_PASSPHRASE);
-    assert_true(rc == 0);
+    rc = ssh_pki_export_privkey_file_format(origkey,
+                                            torture_get_testkey_passphrase(),
+                                            NULL,
+                                            NULL,
+                                            LIBSSH_ECDSA_TESTKEY_PASSPHRASE,
+                                            format);
+    assert_int_equal(rc, 0);
 
     /* Test with invalid passphrase */
     rc = ssh_pki_import_privkey_file(LIBSSH_ECDSA_TESTKEY_PASSPHRASE,
@@ -885,7 +928,7 @@ static void torture_pki_ecdsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == SSH_ERROR);
+    assert_int_equal(rc, SSH_ERROR);
     assert_null(privkey);
 
     rc = ssh_pki_import_privkey_file(LIBSSH_ECDSA_TESTKEY_PASSPHRASE,
@@ -893,15 +936,48 @@ static void torture_pki_ecdsa_write_privkey(void **state)
                                      NULL,
                                      NULL,
                                      &privkey);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(privkey);
 
     rc = ssh_key_cmp(origkey, privkey, SSH_KEY_CMP_PRIVATE);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
 
     SSH_KEY_FREE(origkey);
     SSH_KEY_FREE(privkey);
 }
+
+static void
+torture_pki_ecdsa_write_privkey(void **state)
+{
+    torture_pki_ecdsa_write_privkey_format(state, SSH_FILE_FORMAT_DEFAULT);
+}
+
+#ifdef HAVE_LIBCRYPTO
+static void
+torture_pki_ecdsa_write_privkey_pem(void **state)
+{
+    torture_pki_ecdsa_write_privkey_format(state, SSH_FILE_FORMAT_PEM);
+}
+
+static void
+torture_pki_ecdsa_write_privkey_openssh(void **state)
+{
+    torture_pki_ecdsa_write_privkey_format(state, SSH_FILE_FORMAT_OPENSSH);
+}
+
+static void
+torture_pki_ecdsa_import_export_privkey_base64_pem(void **state)
+{
+    torture_pki_ecdsa_import_export_privkey_base64_format(state,
+                                                          SSH_FILE_FORMAT_PEM);
+}
+
+static void
+torture_pki_ecdsa_import_export_privkey_base64_openssh(void **state)
+{
+    torture_pki_ecdsa_import_export_privkey_base64_format(state,
+                                                          SSH_FILE_FORMAT_OPENSSH);
+}
 #endif /* HAVE_LIBCRYPTO */
 
 static void torture_pki_ecdsa_name(void **state, const char *expected_name)
@@ -913,11 +989,11 @@ static void torture_pki_ecdsa_name(void **state, const char *expected_name)
     (void) state; /* unused */
 
     rc = ssh_pki_import_privkey_file(LIBSSH_ECDSA_TESTKEY, NULL, NULL, NULL, &key);
-    assert_true(rc == 0);
+    assert_int_equal(rc, 0);
     assert_non_null(key);
 
-    etype_char =ssh_pki_key_ecdsa_name(key);
-    assert_true(strcmp(etype_char, expected_name) == 0);
+    etype_char = ssh_pki_key_ecdsa_name(key);
+    assert_string_equal(etype_char, expected_name);
 
     SSH_KEY_FREE(key);
 }
@@ -958,15 +1034,18 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_pubkey_file,
                                         setup_openssh_ecdsa_key_521,
                                         teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64,
-                                        setup_ecdsa_key_256,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64,
-                                        setup_ecdsa_key_384,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64,
-                                        setup_ecdsa_key_521,
-                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_default,
+            setup_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_default,
+            setup_ecdsa_key_384,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_default,
+            setup_ecdsa_key_521,
+            teardown),
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_comment,
                                         setup_ecdsa_key_256,
                                         teardown),
@@ -985,15 +1064,18 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64_whitespace,
                                         setup_ecdsa_key_521,
                                         teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64,
-                                        setup_openssh_ecdsa_key_256,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64,
-                                        setup_openssh_ecdsa_key_384,
-                                        teardown),
-        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_import_privkey_base64,
-                                        setup_openssh_ecdsa_key_521,
-                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_default,
+            setup_openssh_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_default,
+            setup_openssh_ecdsa_key_384,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_default,
+            setup_openssh_ecdsa_key_521,
+            teardown),
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_publickey_from_privatekey,
                                         setup_ecdsa_key_256,
                                         teardown),
@@ -1058,7 +1140,6 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_cert_verify,
                                         setup_ecdsa_key_521,
                                         teardown),
-#ifdef HAVE_LIBCRYPTO
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey,
                                         setup_ecdsa_key_256,
                                         teardown),
@@ -1068,6 +1149,49 @@ int torture_run_tests(void) {
         cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey,
                                         setup_ecdsa_key_521,
                                         teardown),
+#ifdef HAVE_LIBCRYPTO
+        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey_pem,
+                                        setup_ecdsa_key_256,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey_pem,
+                                        setup_ecdsa_key_384,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey_pem,
+                                        setup_ecdsa_key_521,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey_openssh,
+                                        setup_ecdsa_key_256,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey_openssh,
+                                        setup_ecdsa_key_384,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(torture_pki_ecdsa_write_privkey_openssh,
+                                        setup_ecdsa_key_521,
+                                        teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_pem,
+            setup_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_pem,
+            setup_ecdsa_key_384,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_pem,
+            setup_ecdsa_key_521,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_openssh,
+            setup_ecdsa_key_256,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_openssh,
+            setup_ecdsa_key_384,
+            teardown),
+        cmocka_unit_test_setup_teardown(
+            torture_pki_ecdsa_import_export_privkey_base64_openssh,
+            setup_ecdsa_key_521,
+            teardown),
 #endif /* HAVE_LIBCRYPTO */
         cmocka_unit_test(torture_pki_sign_data_ecdsa),
         cmocka_unit_test(torture_pki_fail_sign_with_incompatible_hash),
