@@ -1,3 +1,5 @@
+#include "config.h"
+
 #define LIBSSH_STATIC
 #include <libssh/priv.h>
 #include <libssh/callbacks.h>
@@ -13,30 +15,39 @@
 #endif
 #define NUM_THREADS 100
 
-static void setup(void **state) {
+static int setup(void **state) {
+    int rc;
+
     (void) state;
 
     ssh_threads_set_callbacks(ssh_threads_get_pthread());
-    ssh_init();
+    rc = ssh_init();
+    if (rc != SSH_OK) {
+        return -1;
+    }
+
+    return 0;
 }
 
-static void teardown(void **state) {
+static int teardown(void **state) {
     (void) state;
 
     ssh_finalize();
+
+    return 0;
 }
 
 static void *torture_rand_thread(void *threadid) {
     char buffer[12];
     int i;
-    int r;
+    int ok;
 
     (void) threadid;
 
     buffer[0] = buffer[1] = buffer[10] = buffer[11] = 'X';
     for(i = 0; i < NUM_LOOPS; ++i) {
-        r = ssh_get_random(&buffer[2], i % 8 + 1, 0);
-        assert_true(r == 1);
+        ok = ssh_get_random(&buffer[2], i % 8 + 1, 0);
+        assert_true(ok);
     }
 
     pthread_exit(NULL);
@@ -60,10 +71,13 @@ static void torture_rand_threading(void **state) {
 }
 
 int torture_run_tests(void) {
-    UnitTest tests[] = {
-        unit_test_setup_teardown(torture_rand_threading, setup, teardown),
+    int rc;
+    struct CMUnitTest tests[] = {
+        cmocka_unit_test_setup_teardown(torture_rand_threading, setup, teardown),
     };
 
     torture_filter_tests(tests);
-    return run_tests(tests);
+    rc = cmocka_run_group_tests(tests, NULL, NULL);
+
+    return rc;
 }
